from ._internal import RootCommand, parameter


class Protoc(RootCommand):
    """

    protoc --go_out=. --go_opt=paths=source_relative --go-grpc_out=. --go-grpc_opt=paths=source_relative browser.proto
    """
    __name__ = 'protoc'
    globals = globals()

    @parameter()
    def go_out(self):
        """指定pb文件所在生成目录"""

    @parameter(default="paths=source_relative")
    def go_opt(self):
        """固定值"""

    @parameter(long="--go-grpc_out", default="指定grpc pb文件所在生成目录")
    def go_grpc_out(self):
        """指定grpc pb文件所在生成目录"""

    @parameter(long="--go-grpc_opt", default="paths=source_relative")
    def go_grpc_out(self):
        """固定值"""


protoc = Protoc()


import configparser
import os
import logging
from pip._vendor.platformdirs.windows import Windows


logger = logging.getLogger(__name__)
win = Windows()
DIR_NAME = "felia"
config = configparser.ConfigParser()
ini = win.user_config_path / DIR_NAME / 'felia.ini'
config.read(ini, encoding='utf-8')


class ConfMixin:
    def change_workdir(self):
        section = config[self.__class__.__name__.upper()]
        projects = list(section.items())

        if len(projects) == 1:
            path = projects[0][1]
        else:
            for index, value in enumerate(projects):
                print(index + 1, value[0])

            i = int(input("请选择一个项目")) - 1
            path = projects[i][1]

        print("切换路径:", path)
        os.chdir(path)

    def change_conn(self):
        """切换本地或者连接远程机器

        :return:
        """
        section = config[self.__class__.__name__.upper()]
        projects = list(section.items())
        if len(projects) == 0:
            return

        print(1, "Local")
        for index, value in enumerate(projects, start=2):
            print(index, value[0])
        i = int(input("请选择一个项目"))

        if i == 1:
            import sys
            if sys.platform == "win32" and len(self.pre) == 0:
                self.pre.append('powershell.exe')
                self.pre.append('wsl')
            return

        from fabric import Connection
        import json
        logger.debug(f"projects: {projects}")
        conf = json.loads(projects[i - 2][1])
        conn = Connection(conf["host"],
                          user=conf["user"],
                          port=conf["port"],
                          connect_kwargs={"key_filename": conf["key_filename"]})
        self.pre.clear()
        self.run = conn.sudo


import sys
import logging
import subprocess
from importlib import import_module
import win32api

from .configuration import ConfMixin

logger = logging.getLogger(__name__)
h = logging.StreamHandler()
h.setFormatter(logging.Formatter("%(levelname)s $ %(message)s"))
logger.addHandler(h)
logger.setLevel(logging.INFO)


class BaseCommand:
    """
    cli.para1().para2()(run=False) # 只输出命令，不执行
    等价于
    cli --para1 --para2
    """

    def __init__(self):
        self._params = []
        self.pre = []
        self.args = []
        self.cmd = [self.__name__]
        self.run = subprocess.run
        self.setup()

    def setup(self):
        if sys.platform == "win32":
            self.pre.append('powershell.exe')
            self.pre.append('wsl')

    def __call__(self, *args, **kwargs):
        pass

    def show_commands(self):
        self(run=False)

    def help(self):
        print(f"Usage:  {self.cmd[0]}")
        print()
        for obj in self._params:
            obj.print_help()

    def clear_args(self):
        self.args.clear()

    def execute(self, *args, use_shell=False):
        _args = self.args + list(args)
        self(*_args, use_shell=use_shell)


class RootCommand(BaseCommand, ConfMixin):
    subcommands: list
    globals: dict

    def setup(self):
        super(RootCommand, self).setup()
        if not hasattr(self, 'subcommands'):
            return

        while self.subcommands:
            CMD = self.subcommands.pop()
            obj = CMD(self)
            setattr(self, CMD.__name__, obj)
            obj_name = f"{self.__name__}_{CMD.__name__}".lower()
            self.globals.setdefault(obj_name,
                                    obj)

            if _all := self.globals.get("__all__"):
                _all.append(obj_name)

    def __call__(self, *args, run=True, use_shell=False):
        cmd = self.cmd + list(args)
        logger.info(" ".join(cmd))
        if run:
            if hasattr(self, '__main__'):
                module_name, func_name = getattr(self, '__main__').rsplit(".", 1)
                return getattr(import_module(module_name), func_name)(args)
            elif hasattr(self, "use_shell") and getattr(self, "use_shell") or use_shell:
                # Pycharm python console遇到input会卡住不能交互, 用win32api调用powershell窗口处理
                return win32api.ShellExecute(0, "open", "powershell.exe", "wsl " + " ".join(cmd), "", 1)
            else:
                content = " ".join(self.pre + cmd)
                if self.pre:
                    content = content.replace(" | ", " | wsl ")
                return self.run(content)

    @classmethod
    def subcommand(cls, sub_class):
        if not hasattr(cls, 'subcommands'):
            setattr(cls, 'subcommands', [])
        cls.subcommands.append(sub_class)
        del sub_class


class SubCommand(BaseCommand):

    def __init__(self, root):
        super(SubCommand, self).__init__()
        self.root = root

    def __call__(self, *args, run=True):
        _args = self.args + list(args)
        cmd = self.cmd + _args

        self.root(*cmd, run=run)
        if run:
            self.clear_args()
            self.root.clear_args()









"""

阿里云升级配置后，磁盘手工需要扩容: https://zhuanlan.zhihu.com/p/242619035

Error constructing proxy for org.gnome.Terminal:/org/gnome/Terminal/Factory0: 为 org.gnome.Terminal 调用 StartServiceByName 出错：已到超时限制
https://www.cnblogs.com/ztguang/p/12644799.html
dbus-launch gnome-terminal
"""


from ._internal import RootCommand

__all__ = ["echo_supervisord_conf"]


class EchoSupervisordConf(RootCommand):
    __name__ = "echo_supervisord_conf"
    globals = globals()


echo_supervisord_conf = EchoSupervisordConf()



from ._internal import SubCommand, RootCommand

__all__ = ["pip"]


class Pip(RootCommand):
    __name__ = "pip"
    globals = globals()

    @staticmethod
    def generating_distribution_archives():
        """打包软件

        https://packaging.python.org/en/latest/tutorials/packaging-projects/#generating-distribution-archives
        """
        from .python import python
        python.m("build").execute()

    @staticmethod
    def uploading_the_distribution_archives(repository="pypi"):
        """上传打包文件

        https://packaging.python.org/en/latest/tutorials/packaging-projects/#generating-distribution-archives
        测试环境: https://test.pypi.org/simple/ repository=testpypi
        生产环境: https://pypi.org/simple repository=pypi
        """
        from .python import python
        python.m("twine").execute(f"upload --repository {repository} dist/*", use_shell=True)


@Pip.subcommand
class Config(SubCommand):
    __name__ = "config"


@Pip.subcommand
class Show(SubCommand):
    __name__ = "show"


@Pip.subcommand
class List(SubCommand):
    __name__ = "list"


pip = Pip()


from .base import SubCommand, RootCommand
from .configuration import config


def parameter(short=None, default=None, long=None):
    """TODO short暂时没用途, 可能在未来的doc引用?"""
    def _parameter(func):
        def inner(self, value):
            value = value or default
            option = (long or "--" + func.__name__) + " " + value
            if option not in self.args:
                self.args.append(option)
            return self
        return inner
    return _parameter



from ._internal import SubCommand, RootCommand, parameter

__all__ = ["docker",
           ]


class Ps(SubCommand):
    __name__ = "ps"

    @parameter()
    def all(self):
        """显示所有容器"""

    @parameter()
    def quiet(self):
        """只显示容器ID"""


class Rm(SubCommand):
    __name__ = "rm"


class Rmi(SubCommand):
    """移除一个或者多个镜像"""
    __name__ = "rmi"


class Diff(SubCommand):
    """显示了镜像被实例化成一个容器以来哪些文件受到了影响

    https://docs.docker.com/engine/reference/commandline/diff/
    """
    __name__ = "diff"


class Tag(SubCommand):
    """给一个Docker镜像打标签"""
    __name__ = "tag"


class Stop(SubCommand):
    """干净地终结容器"""
    __name__ = "stop"


class Logs(SubCommand):
    """抓取容器的日志

    源码: https://github.com/docker/compose/blob/v2/cmd/compose/logs.go
    """
    __name__ = "logs"


class Exec(SubCommand):
    """在容器执行bash命令"""
    __name__ = "exec"


class Images(SubCommand):
    """列出所有镜像"""
    __name__ = "images"


class Restart(SubCommand):
    """重启一个或多个容器"""
    __name__ = "restart"


class Version(SubCommand):
    __name__ = "version"


class Start(SubCommand):
    """启动一个或多个停止状态的容器"""
    __name__ = "start"


class Commit(SubCommand):
    """将一个Docker容器作为一个镜像提交"""
    __name__ = "commit"


class Docker(RootCommand):
    __name__ = 'docker'
    globals = globals()
    subcommands = [
            Ps,
            Rm,
            Rmi,
            Tag,
            Stop,
            Diff,
            Logs,
            Exec,
            Start,
            Images,
            Commit,
            Restart,
            Version,
        ]


@Docker.subcommand
class Pull(SubCommand):
    """拉取镜像

    https://docs.docker.com/engine/reference/commandline/pull/
    """
    __name__ = "pull"


@Docker.subcommand
class Search(SubCommand):
    """检索Docker Hub镜像

    Docker实践(第2版) 第95页
    """
    __name__ = "search"


@Docker.subcommand
class Inspect(SubCommand):
    """显示容器的信息

    Return low-level information on Docker objects

    Docker实践(第2版) 技巧30

    https://docs.docker.com/engine/reference/commandline/inspect/
    """
    __name__ = "inspect"

    @parameter(short="-f")
    def format(self):
        """使用Go模板格式化输出

        使用format标志的例子: https://docs.docker.com/engine/reference/commandline/inspect/#examples
        """

    def get_instance_IP_address(self, obj):
        """https://docs.docker.com/engine/reference/commandline/inspect/#get-an-instances-ip-address

        单个使用{{.NetworkSettings.IPAddress}}
        """
        self.format("'{{range .NetworkSettings.Networks}}{{.IPAddress}}{{end}}'")(obj)

    def get_instance_MAC_address(self, obj):
        """https://docs.docker.com/engine/reference/commandline/inspect/#get-an-instances-mac-address"""

    def get_instance_log_path(self, obj):
        """https://docs.docker.com/engine/reference/commandline/inspect/#get-an-instances-log-path"""
        self.format("'{{.LogPath}}'")(obj)

    def get_instance_image_name(self, obj):
        """https://docs.docker.com/engine/reference/commandline/inspect/#get-an-instances-image-name"""
        self.format("'{{.Config.Image}}'")(obj)

    def list_all_port_bindings(self, obj):
        """https://docs.docker.com/engine/reference/commandline/inspect/#list-all-port-bindings"""
        self.format("'{{range $p, $conf := .NetworkSettings.Ports}} {{$p}} -> {{(index $conf 0).HostPort}} {{end}}'")(obj)

    def get_subsection_in_JSON_format(self, obj):
        """https://docs.docker.com/engine/reference/commandline/inspect/#get-a-subsection-in-json-format"""
        self.format("'{{json .Config}}'")(obj)


@Docker.subcommand
class Run(SubCommand):
    """以容器形式运行一个Docker镜像

    源码: https://github.com/docker/compose/blob/v2/cmd/compose/run.go
    """
    __name__ = "run"

    @parameter(short="-i")
    def interactive(self):
        """保持STDIN打开, 用于控制台交互"""

    @parameter(short="-t")
    def tty(self):
        """分配TTY设备, 可以支持终端登录"""

    @parameter(short="-p")
    def publish(self):
        """指定容器包路的端口"""

    @parameter()
    def name(self):
        """分配容器的名称"""

    @parameter(short="-d")
    def detach(self):
        """在后台运行容器和打印容器id"""

    @parameter(short="-l")
    def label(self):
        """设置LABEL元数据"""

    @parameter(short="-v")
    def volume(self):
        """挂载卷, 格式 本地目录:远程目录"""

    # ============= 以下参数不常用 =============================
    @parameter()
    def restart(self):
        """设置重启策略, 默认值'no'。

        策略:

        * no:容器退出时不重启;
        * always:容器退出时总是重启;
        * unless-stopped:总是重启, 不过显示停止除外
        * on-failure[:max-retry]:只在失败时重启
        """

    @parameter(long="--volumes-from")
    def volumes_from(self):
        """挂载指定的数据容器

        Docker实践(第2版) 技巧37
        """


@Docker.subcommand
class Build(SubCommand):
    """构建一个Docker镜像

    文档: https://docs.docker.com/engine/reference/commandline/build/
    """
    __name__ = "build"

    @parameter(short="-t")
    def tag(self):
        """打标签, 格式: 'name:tag'"""

    @parameter(long="--build-arg")
    def build_arg(self):
        """设置运行时的变量值(ARG)"""

    # ======== 以下参数不常用 ========================
    @parameter(long="--no-cache")
    def no_cache(self):
        """构建时不使用缓存"""


docker = Docker()



import logging

from ._internal import SubCommand, RootCommand, parameter


class Go(RootCommand):
    __name__ = 'go'
    globals = globals()


@Go.subcommand
class Mod(SubCommand):
    """项目和包管理"""
    subcmd = "mod"

    @parameter
    def init(self):
        """在当前目录初始化"""


go = Go()



from ._internal import RootCommand, parameter


class SSH(RootCommand):
    __name__ = 'ssh'
    globals = globals()

    @parameter(long="-T")
    def T(self):
        """测试连通性

        ssh -T git@github.com
        """

    def t_github(self):
        self.T().execute("git@github.com")


class SSHKeygen(RootCommand):
    """

    install: apt-get install ssh
    重启ssh服务: /etc/init.d/ssh restart
    """
    __name__ = 'ssh-keygen'
    globals = globals()
    use_shell = True

    @parameter(long="-t")
    def t(self):
        """[dsa | ecdsa | ecdsa-sk | ed25519 | ed25519-sk | rsa]"""


ssh = SSH()
ssh_keygen = SSHKeygen()



from ._internal import RootCommand, parameter


class SphinxQuickstart(RootCommand):
    """初始化sphinx项目
    """
    __name__ = 'sphinx-quickstart'
    __main__ = 'sphinx.cmd.quickstart.main'
    globals = globals()


class SphinxBuild(RootCommand):
    """
    usage: sphinx-build [OPTIONS] SOURCEDIR OUTPUTDIR [FILENAMES...]
    """
    __name__ = 'sphinx-build'
    __main__ = 'sphinx.cmd.build.main'
    globals = globals()

    @parameter(long="-W")
    def W(self):
        """将警告信息转为错误，即遇到警告信息时退出程序"""

    def make_html(self):
        self("source",
             "build")


sphinx_quickstart = SphinxQuickstart()
sphinx_build = SphinxBuild()



import os

import argparse
import configparser
from pip._vendor.platformdirs.windows import Windows

win = Windows()

DIR_NAME = "felia"


def main():
    """python3 -m felia set --cli sphinx --project django --path /root"""
    parser = argparse.ArgumentParser(prog='HELPER',
                                     description="生活和工作的助手程序")
    parser.add_argument('action', metavar='ACTION', type=str, nargs='?',
                        help='设置项目路径')
    parser.add_argument("--cli", dest="cli", action='store',
                        help="命令行接口")
    parser.add_argument('--project', dest='project', action='store',
                        help='项目')
    parser.add_argument('--path', dest='path', action='store',
                        help='项目路径')
    args = parser.parse_args()

    handle = {"set": set_value,
              "get": get_value}
    handle[args.action](args)


def set_value(args):
    config = configparser.ConfigParser()
    felia_path = win.user_config_path / DIR_NAME
    if not os.path.exists(felia_path):
        os.makedirs(felia_path)

    ini = win.user_config_path / DIR_NAME / 'felia.ini'
    config.read(ini)

    section = args.cli.upper()
    if section not in config.keys():
        config.add_section(section)
    config[args.cli.upper()].setdefault(args.project, args.path)

    with open(ini, 'w') as configfile:
        config.write(configfile)


def get_value(args):
    ...


if __name__ == '__main__':
    main()


from ._internal import RootCommand, parameter


class Python(RootCommand):
    __name__ = "python3"
    globals = globals()

    @parameter(long="-m")
    def m(self):
        """以脚本的方式运行指定模块

        -m mod : run library module as a script (terminates option list)
        """


python = Python()


import logging
from pathlib import Path

from felia._internal import RootCommand, SubCommand, change_workdir, config

logger = logging.getLogger('Docker')
h = logging.StreamHandler()
h.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - | %(message)s"))
logger.addHandler(h)
logger.setLevel(logging.INFO)


class Add(SubCommand):
    """增加一个Cobra命令

    cobra-cli add serve
    cobra-cli add config
    cobra-cli add create -p 'configCmd'
    """
    subcmd = "add"


class CobraCli(RootCommand):
    """生成cobra代码

    https://github.com/spf13/cobra-cli/blob/main/README.md"""
    __name__ = 'cobra-cli'
    globals = globals()
    subcommands = [Add, ]


cobra_cli = CobraCli()
cobra_add = getattr(cobra_cli, 'add')

change_workdir(config["COBRA"])